#!/usr/bin/env python3

# PROM reader for 40-pin Raspberry Pi
# Must use level downshifter between
# PROM data pins and Pi inputs.

import time
import RPi.GPIO as GPIO

# Change the value of WriteFile to '1' if you want to
# write the code read out from the PROM to a binary file.

WriteFile=0

# Use Broadcom GPIO pin numbering scheme
GPIO.setmode(GPIO.BCM)

# Address output GPIO pins defined in lowbit to highbit order (A0-A11)
AddressPin = (10,9,11,25,8,7,5,6,12,13,19,26)

# Data input GPIO pins defined in lowbit to highbit order (D0-D7)
DataPin = (14,15,18,17,27,22,23,24)

# CS pins
GPIO.setup (3,GPIO.OUT) # active low CS pin
GPIO.setup (4,GPIO.OUT) # active high CS pin

# Set the chip in standby mode:
GPIO.output(3,1) # high = inactive
GPIO.output(4,0) # low = inactive

# Set Address  pins as outputs
for BitCounter in range (0,12): 
    GPIO.setup (AddressPin[BitCounter],GPIO.OUT)

# Set Data pins as inputs.
for BitCounter in range (0,8):
    GPIO.setup (DataPin[BitCounter],GPIO.IN) 

# If file is enabled, open the file    
if WriteFile==1:       
	file=open("binaryfile.bin","wb")

# Initialise address pointer and checksum	
Address=0
CheckSum=0
    
# Read from all locations of the PROM
while Address<4096: # Number of bytes to read from the PROM

    # Convert Address to ASCII hexadecimal
    AddressAsHex="%0.4X"% Address

    # Print line start address
    print(AddressAsHex,"",end="")

    AsciiLine="" #Clear the ASCII line buffer

    # Output 16 columns per line
    for ColumnCount in range (0,16):
    
        # Output the current address to individual GPIO output pins        
        BitMask=1
        for BitCounter in range (0,12):
            if (Address & BitMask==BitMask):
                GPIO.output((AddressPin[BitCounter]),1)
            else:
                GPIO.output((AddressPin[BitCounter]),0)
            BitMask=(BitMask<<1)

        # Enable PROM data outputs
        GPIO.output(3,0) # low = active
        GPIO.output(4,1) # high = active

        # Might need a small delay here if running from the
        # command line on a fast Pi
	# time.sleep(0.01)
		
        # Read the individual databits and assemble them in 'DataByte'
        DataByte=0
        BitValue=1        
        for BitCounter in range (0,8):
            if (GPIO.input(DataPin[BitCounter])==1):
                DataByte=(DataByte+BitValue)
            BitValue=(BitValue<<1)    

        # Disable PROM data outputs
        GPIO.output(3,1) # high = inactive
        GPIO.output(4,0) # low = inactive

        # If file write enabled, send the byte out to the file
        if WriteFile==1:
            DataForFile=bytes([DataByte])
            file.write(DataForFile)
			
        # Add DataByte value to 16-bit checksum
        CheckSum=CheckSum+DataByte

        # Convert Data to ASCII hexadecimal        
        DataAsHex="%0.2X" % DataByte
        
        # Print the data
        print(DataAsHex,"",end="")

        # If DataByte holds a printable character code, add
        # the corresponding ASCII character to the ASCII buffer
        if (DataByte>31 & DataByte <128):
            AsciiLine=(AsciiLine+(chr(DataByte)))
        # Otherwise just add '.'    
        else:
            AsciiLine=(AsciiLine+".")

        Address=Address+1
                              
    print (AsciiLine,end="")

    print("")

# Convert 16-bit checksum to ASCII hexadecimal
CheckSum=(CheckSum & 0xFFFF)
CheckSumAsHex="%0.4X"% CheckSum

# Print the 16-bit checksum
print("")
print("Checksum =",CheckSumAsHex)

# If file write was enabled, close the file
if WriteFile==1:
	file.close()

GPIO.cleanup()    
